package org.eclipse.ecf.example.sc.core;

import java.util.HashMap;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ecf.core.ContainerFactory;
import org.eclipse.ecf.core.ContainerInstantiationException;
import org.eclipse.ecf.core.IContainer;
import org.eclipse.ecf.core.identity.ID;
import org.eclipse.ecf.core.identity.IDFactory;
import org.eclipse.ecf.core.identity.IDInstantiationException;
import org.eclipse.ecf.core.util.ECFException;
import org.eclipse.ecf.datashare.IChannel;
import org.eclipse.ecf.datashare.IChannelContainer;
import org.eclipse.ecf.datashare.IChannelListener;
import org.eclipse.ecf.example.sc.core.model.preferences.PreferenceConstants;
import org.eclipse.ecf.example.sc.core.model.preferences.Preferences;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;


/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	private boolean connected;
	// The plug-in ID
	public static final String PLUGIN_ID = "ShareCode";

	// The shared instance
	private static Activator plugin;

	@SuppressWarnings("unused")
	private IChannelListener presenceChannelListener;

	private IChannel presenceChannel;
	
	private Preferences preferences;
	
	/**
	 * The constructor
	 */
	public Activator() {
		plugin = this;
		plugin.setPreferences(new Preferences());
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	public IChannel intializePresenceSession(IChannelListener clistener) throws ECFException {
		if(presenceChannel == null){
			IContainer presenceContainer = null;
			try {
				presenceContainer = ContainerFactory.getDefault().createContainer(getPreferences().getContainerType());
			
				IChannelContainer channelContainer = (IChannelContainer) presenceContainer.getAdapter(IChannelContainer.class);
	
				ID channelID = IDFactory.getDefault().createID(channelContainer.getChannelNamespace(),getPreferences().getChannelId());
				
				presenceChannel = channelContainer.createChannel(channelID, clistener, new HashMap());
				
				this.presenceChannelListener = clistener;
	
				presenceContainer.connect(IDFactory.getDefault().createID(presenceContainer.getConnectNamespace(),getPreferences().getTargetServer()), null);
				
				return presenceChannel;
				
			
			} catch (ContainerInstantiationException e) {
//				 TODO Tratar LOG
				presenceChannel = null;
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e.getLocalizedMessage(), e));
				throw new ECFException(e);
			} catch (IDInstantiationException e) {
				presenceChannel = null;
//				 TODO Tratar LOG
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e.getLocalizedMessage(), e));
				throw new ECFException(e);
			} catch (ECFException e) {
				presenceChannel = null;
//				 TODO Tratar LOG
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e.getLocalizedMessage(), e));
				throw new ECFException(e);
			}
		}
		return presenceChannel;
	
		
		
	}
	

	

	public IChannel getPresenceChannel() {
		return presenceChannel;
	}


	public void setListenerActive(boolean b) {
		// TODO identificar a necessidade desse metodo
		
	}

	public boolean isConnected() {
		return connected;
	}

	public void setConnected(boolean connected) {
		this.connected = connected;
	}

	public Preferences getPreferences() {
		return preferences;
	}

	public void setPreferences(Preferences preferences) {
		this.preferences = preferences;
	}
}
